-- -- Invas0res -- --
-- Author   : Leo M. Cabrera
-- E-mail   : leo28c@gmail.com
-- Date     : March 18, 2007
-- Version  : Beta 4
-- File     : (7/7) alienfuncs.lua
-- Does     : Alien-related variables and functions functions

function loadAliens()
	-- Set the alien's width and height
	aliens.imgW = aliens.img:width()
	aliens.imgH = aliens.img:width()

	-- Position for the alien's guns
	alienGun = {x = 21, y = 40}
	
	if level == 5 then
		numAliens = 12
	elseif level == 10 then
		numAliens = 24
	elseif level == 15 then
		numAliens = 36
	elseif level == 30 then
		numAliens = 54
	else
		-- Number of aliens
		numRows   = 5
		numAliens = numRows * math.min(4, level) -- 4 maximum
	end
	
	-- Change according to difficulty level
	if level == 5 or level == 10 or level == 15 or level == 30 then
		if diffLevel == 3 then numAliens = numAliens * 1.5 end
	end
	
	-- Init the layer (bosses have al the same w/h)
	if level == 5 or level == 10 or level == 15 or level == 30 then
		aliens.layer = Image.createEmpty(300,
			200)
	else
		-- TODO: Fix error!
		aliens.layer = nil
		aliens.layer = Image.createEmpty(numRows * aliens.imgW * 2, numAliens/numRows * aliens.imgH)
	end
	
	-- The alien explosions (used to draw the fire around them after they die)
	aliens.explosions = {}
	
	-- Initialize them
	for ae = 0, numAliens-1 do
		aliens.explosions[ae] = {}
		aliens.explosions[ae].on = false
		if _LPN_ == true then
			aliens.explosions[ae].timer = Timer()
		else
			aliens.explosions[ae].timer = Timer.new()
		end
	end
	
	-- Aliens movement timer
	if _LPN_ == true then
		aliensTimer = Timer()
	else
		aliensTimer = Timer.new()
	end
	
	aliensTimer:start()
	aliensNextMove = aliensTimer:time()
	
	-- Alien's timer increment
	if level <= 5 then
		atInc = 1500
	elseif level <= 10 then
		atInc = 1000
	elseif level <= 15 then
		atInc = 500
	elseif level < 20 then
		atInc = 250
	elseif level < 25 then
		atInc = 200
	elseif level <= 30 then
		atInc = 100
	end
		
	-- Change it according to difficulty level
	if diffLevel == 1 then atInc = atInc * 1.3
	elseif diffLevel == 3 then atInc = atInc / 2 end
	
	aliveAliens = numAliens
	
	-- X-axis of the aliens (the same as the top-left-most alien)
	-- plus the min. and max. values
	aliens.lyrXpos = { min = 0,
	        		   max = scrWidth - aliens.layer:width() + aliens.imgW,
					   cur = scrWidth/2 - aliens.layer:width()/2}
					
	-- The Y-axis; unlike the above, this one is in rows and not in pixels
	aliens.lyrYpos = { max = 4 - level,
					   min = 0,
					   cur = 0 }
end -- loadAliens()


function drawAliens()

	-- Load regular alien image if back at level 1
	if level == 1 then aliens.img  = Image.load("images/alien.png") end
	
	-- Load superalien image once on level 6
	if level == 6 then aliens.img  = Image.load("images/superalien.png") end
	
	-- Load hyperalien image once on level 16
	if level == 16 then aliens.img  = Image.load("images/hyperalien.png") end
	
	-- Alien's direction
	aliens.xDir  =  1
	aliens.yDir  =  0
	
	-- Set the alien speeds for the various levels
	if level > 15 then
		-- We don't want it TOO fast now; plus they've got shields
		aliens.speed =  20
	elseif level > 10 then
		-- We don't want it TOO fast now; plus they've got shields
		aliens.speed =  20 + level/4
	elseif level > 5 then
		aliens.speed =  20 + level*1.5
	else
		aliens.speed =  20 + level
	end
	
	-- Blit bosses if on a boss level
	if level == 5 then
		aliens.layer:blit(0, 0, Image.load("images/boss5.png"))
	elseif level == 10 then
		aliens.layer:blit(0, 0, Image.load("images/boss10.png"))
	elseif level == 15 then
		aliens.layer:blit(0, 0, Image.load("images/boss15.png"))
	elseif level == 30 then
		aliens.layer:blit(0, 0, Image.load("images/boss30.png"))
	else
	
		-- Blit all them aliens
		for x = 0, numRows-1 do
			for y = 0, (numAliens/numRows)-1 do
				local p = y*numRows+x
				-- Set its x/y and make it be alive
				aliens[p] = { x   = x * aliens.imgW * 2,
				      y   = y * aliens.imgH,
				      hp  = 1,
				      hasShot = false }

				aliens.layer:blit(aliens[p].x,
					aliens[p].y, aliens.img)
				      
				-- Give them shields on levels 10+
				if level > 20 then
					aliens[p].hp = 4
					aliens.layer:blit(aliens[p].x,
						aliens[p].y, aliens.shieldImg[aliens[p].hp-1])
				elseif level > 15 then
					aliens[p].hp = 3
					aliens.layer:blit(aliens[p].x,
						aliens[p].y, aliens.shieldImg[aliens[p].hp-1])
				elseif level > 10 then
					aliens[p].hp = 2
					aliens.layer:blit(aliens[p].x,
						aliens[p].y, aliens.shieldImg[aliens[p].hp-1])
				end
			end
		end
	end
end -- drawAliens()


function processAliens()
					
	-- Only move them every 1.5 seconds
	if aliensNextMove <= aliensTimer:time() then
		local changeYpos = false
		
		-- Change direction
		if aliens.lyrXpos.cur >= aliens.lyrXpos.max then
			aliens.xDir = -1
			changeYpos = true
		elseif aliens.lyrXpos.cur <= aliens.lyrXpos.min then
			aliens.xDir =  1
			changeYpos = true
		end
		
		-- HERE: Crash?!
		-- Only change it if level's less than or equal to 4 (there's no space otherwise)
		if level < 4 and changeYpos then
			-- Change direction
			if aliens.lyrYpos.cur >= aliens.lyrYpos.max then aliens.yDir = -1 end

			if aliens.lyrYpos.cur <= aliens.lyrYpos.min then aliens.yDir = 1 end
			
			-- Move the aliens' internal position
			for a = 0, numAliens-1 do
				aliens[a].y = aliens[a].y + aliens.imgH * aliens.yDir
			end
			
			-- Move the layer (alien images)
			aliens.lyrYpos.cur = aliens.lyrYpos.cur + aliens.yDir
		end
		
		-- Movement speed; 10px range random
		local mov = math.random(aliens.speed - 5, aliens.speed + 5)
		
		-- Move the aliens layer
		aliens.lyrXpos.cur = aliens.lyrXpos.cur + aliens.xDir * mov

  		-- Move each individual alien if not in a boss battle
  		if not (level == 5 or level == 10 or level == 15 or level == 30) then
 			for a = 0, numAliens-1 do
				aliens[a].x = aliens[a].x + aliens.xDir * mov
		 	end
		end
		
		-- Add to the timer
		aliensNextMove = aliensTimer:time() + atInc
		
		if level == 5 or level == 10 or level == 15 or level == 30 then
			-- Holds the current bullet
			local b = 0

			-- Loop through all the lasers and shoot a bullet
  			for l = 32, 262, 23 do
	  			if math.random(0, 4) >= 1 then
					alienBullets[b] = { active = true,
								 x = l + aliens.lyrXpos.cur,
								 y = 183 + aliens.lyrYpos.cur }
				end
				b = b + 1
	  		end
			
		else
			alienShoot()
			
			for a = 0, numAliens-1 do
				aliens[a].hasShot = false
			end
		end
	end
end -- processAliens()


function processExplosions()
	-- Process all explosions
	for a = 0, numAliens-1 do
		-- If it is on and it's been that way for 1 second or more,
		if aliens.explosions[a].on and
		   aliens.explosions[a].timer:time() >= 1000 then
			-- Turn it off and reset its timer
			aliens.explosions[a].on = false
			aliens.explosions[a].timer:reset()
			
			-- Remove the explosion from the screen (fill its spot transparent)
			aliens.layer:fillRect(
				aliens[a].x - aliens.lyrXpos.cur,
				aliens[a].y - aliens.lyrYpos.cur * aliens.imgH,
				aliens.imgW,
			  	aliens.imgH,
				transp)
		end
			
	end
end -- processExplosions()


function alienShoot()
	for b = 0, alienBullets.max do
		if not alienBullets[b].active then
			-- Get a random alien
			repeat
				a = math.random(0, numAliens-1)
			until aliens[a].hp > 0
	
			-- Set its bullet to inactive
			alienBullets[b].active = true
			
			local x = math.random(alienGun.x - 3, alienGun.x + 3)
			local y = alienGun.y
	
			-- Set the new position
			alienBullets[b].x = aliens[a].x + x
			alienBullets[b].y = aliens[a].y + y
		end
	end
	
	-- TODO: Fix the 2-bullet thing
--~ 	for b = 0, alienBullets.max do
--~ 		-- Shoot!
--~ 		if not alienBullets[b].active then
--~ 			-- Get a random alien
--~ 			repeat
--~ 				a = math.random(0, numAliens-1)
--~ 			until aliens[a].hp > 0
--~ 	
--~ 			if aliens[a].hasShot == false then
--~ 				-- Set its bullet to inactive
--~ 				alienBullets[b].active = true
--~ 	
--~ 				local x = math.random(alienGun.x - 3, alienGun.x + 3)
--~ 	
--~ 				-- Set the new position
--~ 				alienBullets[b].x = aliens[a].y + x
--~ 				alienBullets[b].y = aliens[a].y + alienGun.y
--~ 			
--~ 				-- Set this variable so I know which alien has a bullet under it;
--~ 				-- so there aren't 2 bullets from the same alien
--~ 				aliens[a].hasShot = true
--~ 			end
--~ 		end
--~ 	end
end -- alienShoot()
