-- -- Invas0res -- --
-- Author   : Leo M. Cabrera
-- E-mail   : leo28c@gmail.com
-- Date     : March 18, 2007
-- Version  : Beta 4
-- File     : (6/7) render.lua
-- Does     : Game-related graphics functions

-- Load all of the colors
if _LPN_ == true then
	white = Color(255,255,255)
	black = Color(  0,  0,  0)
	red   = Color(255,  0,  0)
	yellow   = Color(255,255,  0)
	transp= Color(  0,  0,  0,255)
else
	white = Color.new(255,255,255)
	black = Color.new(  0,  0,  0)
	red   = Color.new(255,  0,  0)
	yellow   = Color.new(255,255,  0)
	transp= Color.new(  0,  0,  0, 0)
end

function updateScreen()
	if _LPN_ == true then
		screen:waitVblankStart()
		screen:flip()
	else
		screen.waitVblankStart()
		screen.flip()
	end
end

-- Same as Image:fillRect() but it doesn't fill the rectangle;
-- it just draws the border
function drawRect(img, x, y, w, h, color)
	-- Draw a rect with the color
	img:fillRect(x, y, w, h, color)
	-- Fill the middle with transparent
	img:fillRect(x+1, y+1, w-2, h-2, transp)
end

function createBackground()
	-- Star (white dot)
	star = Image.createEmpty(1, 1)
	star:pixel(0,0, white)

	-- Fill with color yellow
	background:fillRect(0, 0, scrWidth, scrHeight, black)

	-- Put 500 black dots on random locations
	for s = 1, 500 do
		x = math.random(scrWidth)
		y = math.random(scrHeight)
	 	background:blit(x, y, star)
	end
end -- createBackground()


function placeShields()
	rWidth = scrWidth / shieldsPos.w
	for s = 1, numShields do
		shieldsLyr:blit(s * (scrWidth / (numShields+1)) - shieldsPos.w/2,
			shieldsPos.y,
			shieldImg)
	end
end -- placeShields()


function loadGraphics()

	-- Make an instance of the default font
	msFont = Font.createMonoSpaced()

	-- Load background image
	background = Image.createEmpty(scrWidth, scrHeight)

	-- Logo to be displayed at the beginning of each level
	logo = Image.load("images/logo.png")

	-- Status images (pause, success, defeat, and game over)
	boredImg = Image.load("images/bored.png")
	youWin  = Image.load("images/you_win.png")
	youLose = Image.load("images/you_lose.png")
	gameOver= Image.load("images/game_over.png")
	
	-- Load the difficulty level images
	diffLvlImg = {}
	diffLvlImg[1]   = Image.load("images/diffLv1.png")
	diffLvlImg[2]   = Image.load("images/diffLv2.png")
	diffLvlImg[3]   = Image.load("images/diffLv3.png")
	
	ship = {}

	-- Load ship image
	ship.img   = Image.load("images/ship.png")
	ship.w = ship.img:width()
	ship.h = ship.img:height()
	
	-- Load shield
	ship.shieldImg = {}
	ship.shieldImg[1]   = Image.load("images/pShield1.png")
	ship.shieldImg[2]   = Image.load("images/pShield2.png")
	ship.shieldImg[3]   = Image.load("images/pShield3.png")
	
	-- Load explosion
	ship.expImg   = Image.load("images/fire-32.png")
	
	-- Life image (shipImg[1] scaled down basically)
	lifeImg      = Image.load("images/life.png")

	-- Load shield image
	shieldImg  = Image.load("images/shield.png")

	-- Shields layer
	shieldsLyr = Image.createEmpty(scrWidth, scrHeight)

	-- Load user, alien, and boss bullet images
	bulletImg   = Image.load("images/bullet.png")
	aBulletImg  = Image.load("images/alienbullet.png")
	bBulletImg  = Image.load("images/bossbullet.png")

	-- Load regular alien image (will change to superalien at some point)
	aliens.img  = Image.load("images/alien.png")
	
	-- Alien explosion image
	aliens.expImg = Image.load("images/fire-48.png")
	aliens.bExpImg= Image.load("images/fire-boss.png")
	
	-- Load shields for the aliens
	aliens.shieldImg = {}
	aliens.shieldImg[1]  = Image.load("images/aShield1.png")
	aliens.shieldImg[2]  = Image.load("images/aShield2.png")
	aliens.shieldImg[3]  = Image.load("images/aShield3.png")
end


function render(specialStatus)

	-- Blit the background
	screen:blit(0, 0, background)
	
	-- Special rendering for intro screen
	if specialStatus == "intro" then
		-- Print the version string
		screen:print(scrWidth/2, 150, verStr, white)
	
		screen:print(scrWidth/2, 165, "Get ready!", white)
		
		screen:print(scrWidth/2, 180, "Level "..level, white)
		
		-- TODO: Redundant code? (below)
		-- Print the logo
		screen:blit(scrWidth/2 - logo:width()/2, 50, logo)
	
		screen:blit(0, 0, aliens.img)
		screen:blit(scrWidth - aliens.img:width(), 0, aliens.img)
		screen:blit(scrWidth/2 - ship.w/2,
			scrHeight - ship.h, ship.img)
			
		-- Update the screen
		updateScreen()
		
		return
	
	-- Special rendering for intro/difficulty selection screen
	elseif specialStatus == "intro2" then
		screen:print(scrWidth/3, 150, "Use L/R buttons to change difficulty", white)
		screen:print(scrWidth/3, 165, "and then press X to continue.", white)
		screen:blit(scrWidth/2 - diffLvlImg[diffLevel]:width()/2,
			180, diffLvlImg[diffLevel])
		
		-- TODO: Redundant code? (above)
		-- Print the logo
		screen:blit(scrWidth/2 - logo:width()/2, 50, logo)
	
		screen:blit(0, 0, aliens.img)
		screen:blit(scrWidth - aliens.img:width(), 0, aliens.img)
		screen:blit(scrWidth/2 - ship.w/2,
			scrHeight - ship.h, ship.img)
			
		-- Update the screen
		updateScreen()
		
		return
	end
	
	-- Special rendering for both intro screens
	if specialStatus == "intro" or specialStatus == "intro2" then
		-- Print the logo
		screen:blit(scrWidth/2 - logo:width()/2, 50, logo)
	
		screen:blit(0, 0, aliens.img)
		screen:blit(scrWidth - aliens.img:width(), 0, aliens.img)
		screen:blit(scrWidth/2 - ship.w/2,
			scrHeight - ship.h, ship.img)
			
		-- Update the screen
		updateScreen()
		
		return
	end

	-- Draw the shields layer
	screen:blit(0, 0, shieldsLyr)

	-- Blit the aliens
	screen:blit(aliens.lyrXpos.cur, aliens.lyrYpos.cur * aliens.imgH,
		aliens.layer)

	-- Draw the player's bullet
	for b = 1, maxBullets do
		if playerBullet[b].active then
			screen:blit(playerBullet[b].x, playerBullet[b].y, bulletImg)
		end
	end

	-- Draw the alien bullets
	for b = 0, alienBullets.max do
		if alienBullets[b].active then
			if level == 5 or level == 10 or level == 15 or level == 30 then
				screen:blit(alienBullets[b].x, alienBullets[b].y, bBulletImg)
			else
				screen:blit(alienBullets[b].x, alienBullets[b].y, aBulletImg)
			end
		end
	end

	-- Blit the ship
	if ship.health == 0 then screen:blit(ship.x, ship.y, ship.expImg)
	else screen:blit(ship.x, ship.y, ship.img) end
	
	-- Blit the shield
	if ship.health > 1 then screen:blit(ship.x, ship.y, ship.shieldImg[ship.health-1]) end
	
	-- Print boss health if on a boss battle level
	if level == 5 or level == 10 or level == 15 or level == 30 then
		-- Red rectangle. It's highet during boss battle
		drawRect(screen, 8, 8, 90, 31, red)
		
		screen:print(10, 30, "Boss: " ..
			math.ceil(aliveAliens/numAliens*100) .. '%', red)
	end
		
	-- Red rectangle. It's highet during
	drawRect(screen, 8, 8, 90, 21, red)

	-- Print score and level	
	screen:print(10, 10, "Score: "..score, red)
	screen:print(10, 20, "Level: "..level, red)
	
	-- Print the lives
	drawRect(screen, scrWidth - lives*20 - 2, livesPos.y, lives*20, 20, red)
	for l = 1, lives do
		screen:blit(scrWidth - l*20 - 2, livesPos.y, lifeImg)
	end
	--screen:print(l - 10, livesPos.y, "Lives", yellow)

	-- TODO: Make this in win() and not use a boolean here?
	-- Blit the "You Win!" message
	if specialStatus == "win" then
		screen:blit(scrWidth/2 - youWin:width()/2, scrHeight/2 - youWin:height()/2,
			youWin)
			
	elseif specialStatus == "lose" then
		screen:blit(scrWidth/2 - youWin:width()/2, scrHeight/2 - youWin:height()/2,
			youLose)
			
	elseif specialStatus == "gameover" then
		screen:blit(scrWidth/2 - youWin:width()/2, scrHeight/2 - youWin:height()/2,
			gameOver)
		screen:print(scrWidth/2, scrHeight/2 - youWin:height()/2,
			"Press cross to continue...", red)
			
	elseif specialStatus == "ask exit" then
		-- Print out some info text
		screen:blit(scrWidth/2 - boredImg:width()/2,
			scrHeight/2 - boredImg:height()/2, boredImg)
	end

	-- Update the screen
	updateScreen()

end -- render()
