-- -- Invas0res -- --
-- Author   : Leo M. Cabrera
-- E-mail   : leo28c@gmail.com
-- Date     : March 18, 2007
-- Version  : Beta 4
-- File     : (4/7) vars.lua
-- Does     : Contains the gameplay variables, a function to reset them, and the
--            function to display the intro

debug = true

-- Current level
level = 1

-- Player's score
score = 0

-- Difficulty level (1-3)
diffLevel = 2

-- You start with 3 lives
lives = 3
stLives = lives

-- Width and height of screen
scrWidth     = 480
scrHeight    = 272

-- TODO: X not needed...
livesPos = { y = 10 }

-- Program version
verStr = "Version 1.0"

-- Aliens table
aliens = {}

-- An alien explosion lasts 1 second
explosionDuration = 1000

function displayIntro(status)
	-- Create a random space background and put it on the screen
	createBackground()
		
	if status == "select difficulty" then
		local lPressed = false
		local rPressed = false
		
		while true do
			-- Update the controls
			pad = Controls.read()
			
			-- Did the user press the shoulder buttons? If so, change diff. level
			-- Left gun shoot
			if pad:l() then
				if not lPressed then
					lPressed = true
					if diffLevel > 1 then diffLevel = diffLevel - 1 end
				end
			else
				lPressed = false
			end
			
			if pad:r() then
				if not rPressed then
					rPressed = true
					if diffLevel < 3 then diffLevel = diffLevel + 1 end
				end
			else
				rPressed = false
			end
			
			-- Exit loop if we're done
			if pad:cross() then break end
			
			-- Display instructions intro
			render("intro2")
			
			if _LPN_ == true then Events.Poll() end
		end
	
	else
		-- Display regular intro
		render("intro")
		System.sleep(2500)
	end
end -- displayIntro()


function resetGameplay()
		
		
	--- PLAYER VARIABLES ---

		ship.x = scrWidth/2 - ship.w/2	-- X-axis
		ship.y = scrHeight - ship.h		-- Y-axis
		ship.speed = 4				-- Speed
		ship.health = 1				-- ship.health
				 
		-- If level is greater than 5, make the speed move 50% faster
		if level > 5 then ship.speed = ship.speed*1.5 end
		
		-- Add the shields
		if level > 5 then ship.health = ship.health + 1 end
		if level > 10 then ship.health = ship.health + 1 end
		if level > 20 then ship.health = ship.health + 1 end
							
				 
	--- OTHER GAME VARIABLES ---
	
		-- Position for the 2 ship guns
		gun = {}
		gun[1] = {x =  5, y = 15}
		gun[2] = {x = 34, y = 15}
		
	
	--- BULLET VARIABLES
	
		-- Bullet travel speed
		bulletSpeed = level+2
		
		-- We can't make it too hard; so let's divide the # of bullets
		if level > 10 then bulletSpeed = bulletSpeed / 1.5 end
		
		if level == 15 then bulletSpeed = bulletSpeed / 1.1 end
		
		-- Change it according to difficulty level
		if diffLevel == 1 then bulletSpeed = bulletSpeed / 1.3
		elseif diffLevel == 3 then bulletSpeed = bulletSpeed * 2 end
		
		-- # of player bullets varies between levels
		if level < 5 then maxBullets = 1
		elseif level < 10 then maxBullets = 2
		elseif level < 20 then maxBullets = 3
		elseif level <= 30 then maxBullets = 4
		end
		
		-- Init array
		playerBullet = {}
	
		-- Init the bullets
		-- 2 + # of alien bullets
		for b = 1, maxBullets do
			playerBullet[b] = {x = 0, y = 0, active = false}
		end
		
		-- Bullet array
		alienBullets = {}

		-- 12 for the bosses (they both have 12 lasers)
		if level == 5 or level == 10 or level == 15 then alienBullets.max = 12
		
		-- Maximum number of bullets (7 is absolute maximum)
		else alienBullets.max = math.min(7, level) end
		
		if level > 15 then alienBullets.max = alienBullets.max * 1.25 end
		
		for b = 0, alienBullets.max do
		    alienBullets[b] = {x = 0, y = 0, active = false}
		end
		
	
	shieldsPos = {x = 0, y = 190, w = 56, h = 32}
	numShields = 5
end

function cleanUp()
	-- Nil-out images
	msFont = nil
	background = nil
	logo = nil
	boredImg = nil
	youWin  = nil
	youLose = nil
	gameOver= nil
	diffLvlImg = nil
	ship.img   = nil
	ship.shieldImg = nil
	ship.expImg   = nil
	lifeImg      = nil
	shieldImg  = nil
	shieldsLyr = nil
	bulletImg   = nil
	aBulletImg  = nil
	bBulletImg  = nil
	aliens.img  = nil
	aliens.expImg = nil
	aliens.bExpImg= nil
	aliens.shieldImg = nil
end
